@@ -95,6 +95,35 @@ def to_string(definition: dict, param: str) -> str:
     return f'fmt.Sprint({param})'
 
 
+def default_value(definition: dict, swagger: dict) -> str:
+    if 'schema' in definition:
+        # work-around for parameters
+        return default_value(definition['schema'])
+
+    ref = definition.get('$ref')
+    if ref is not None:
+        type_name = ref.split('/')[-1]
+        child = swagger.get('definitions', {}).get(type_name, {})
+        if child.get("type", 'object') == 'object':
+            return type_name + "{}"
+        return default_value(child, swagger)
+
+    type_name = definition['type']
+
+    if type_name == 'string':
+        fmt = definition.get('format')
+        if fmt == 'date-time':
+            return 'time.Time{}'
+        return '""'
+    if type_name == 'boolean':
+        return 'false'
+    if type_name == 'array':
+        return 'nil'
+    if type_name == 'number' or type_name == 'integer':
+        return '0'
+    return type_name + '"{}"'
+
+
 def path(text: str) -> str:
     return re.sub(r'{(.*?)}', ':\\1', text)
 
@@ -144,6 +173,15 @@ def comment(text: str) -> str:
     return '\n'.join('// ' + line for line in text.splitlines())
 
 
+def resolve(definition: dict, swagger: dict):
+    if 'schema' in definition:
+        return resolve(definition['schema'], swagger)
+    if '$ref' in definition:
+        name = definition['$ref'].split('/')[-1]
+        return swagger['definitions'][name]
+    return definition
+
+
 def main():
     parser = ArgumentParser(description='Zombie swagger 2.0')
     parser.add_argument('--swagger', '-s', type=Path, default=(Path.cwd() / "swagger.yaml"),
@@ -155,6 +193,8 @@ def main():
     parser.add_argument('--credential', '-c', type=str, default='interface{}', help='Credential type')
     args = parser.parse_args()
 
+    patterns_cache = defaultdict(lambda: f"pattern{len(patterns_cache)}")
+
     env = Environment(loader=FileSystemLoader(args.templates))
     env.filters['map_type'] = map_type
     env.filters['label'] = label
@@ -164,9 +204,13 @@ def main():
     env.filters['comment'] = comment
     env.filters['from_string'] = from_string
     env.filters['to_string'] = to_string
+    env.filters['default_value'] = lambda x: default_value(x, swagger)
     env.filters['secured'] = lambda x: len(x.get('security', [])) > 0
     env.filters['sec_def'] = lambda x: swagger['securityDefinitions'][x]
+    env.filters['resolve'] = lambda x: resolve(x, swagger)
+    env.filters['patterns'] = lambda x: patterns_cache[x]
     env.filters['has_payload'] = lambda x: any(param for param in x.get('parameters', []) if param['in'] == 'body')
+    env.filters['is_ref_to_type'] = lambda x: '$ref' in x or '$ref' in x.get('schema', {})
     env.filters['has_query_params'] = lambda x: any(
         param for param in x.get('parameters', []) if param['in'] == 'query')
     # filter params by place (body, query, ...)
@@ -186,6 +230,7 @@ def main():
     enums = tuple(iter_enums(swagger))
 
     base_file = args.output / "interfaces.go"
+    validations_file = args.output / "validations.go"
     client_file = args.output / "client" / "client.go"
     server_file = args.output / "server" / "server.go"
     base_file.parent.absolute().mkdir(parents=True, exist_ok=True)
@@ -202,6 +247,9 @@ def main():
             for tag in endpoint.get('tags', []):
                 tags[tag].add(endpoint['operationId'])
 
+    type_aliases = dict((name, definition) for (name, definition) in swagger.get('definitions', {}).items()
+                        if 'enum' not in definition and definition.get('type', '') != 'object')
+
     base_file.write_text(env.get_template('base.jinja2').render(
         header=f"// Code generated by simple-swagger {' '.join(sys.argv[1:])} DO NOT EDIT.",
         swagger=swagger,
@@ -209,12 +257,28 @@ def main():
         credential_type=security_type,
         api_package=api_package,
         enums=enums,
+        type_aliases=type_aliases,
         has_security=len(
             swagger.get('securityDefinitions', {})) > 0,
         tags=tags,
         methods=methods,
     ))
 
+    validations_file.write_text(env.get_template('validations.jinja2').render(
+        header=f"// Code generated by simple-swagger {' '.join(sys.argv[1:])} DO NOT EDIT.",
+        swagger=swagger,
+        package=package,
+        credential_type=security_type,
+        api_package=api_package,
+        enums=enums,
+        type_aliases=type_aliases,
+        has_security=len(
+            swagger.get('securityDefinitions', {})) > 0,
+        tags=tags,
+        methods=methods,
+        patterns_cache=patterns_cache,
+    ))
+
     server_file.write_text(env.get_template('server.jinja2').render(
         header=f"// Code generated by simple-swagger {' '.join(sys.argv[1:])} DO NOT EDIT.",
         swagger=swagger,
@@ -237,6 +301,7 @@ def main():
 
     files = [
         str(base_file),
+        str(validations_file),
         str(client_file),
         str(server_file)
     ]
